import requests
from tenacity import retry, wait_random_exponential


HOSTNAME = "10.22.0.110:5000"

BASE_URL = f"http://{HOSTNAME}"


def get_networks():
    res = requests.get("/".join([BASE_URL, "cuwbnets"]))
    return res.json()


def start_network(name):
    res = requests.post(
        "/".join(([BASE_URL, "cuwbnets", name])), json={"action": "start"}
    )
    return res.json().get("status") == "success"


def stop_network(name):
    res = requests.post(
        "/".join(([BASE_URL, "cuwbnets", name])), json={"action": "stop"}
    )
    return res.json().get("status") == "success"


def is_network_running(name):
    res = requests.get("/".join(([BASE_URL, "cuwbnets", name])))
    cuwbnets = res.json().get("cuwbnets")
    if cuwbnets:
        return cuwbnets[0].get("running")
    return False


class FailedToStart(Exception):
    def __init__(self, name):
        super().__init__(f"Failed to start `{name}` cuwbnet")


class FailedToStop(Exception):
    def __init__(self, name):
        super().__init__(f"Failed to stop `{name}` cuwbnet")


@retry(wait=wait_random_exponential(multiplier=1, max=10))
def ensure_network_is_running(name):
    if not is_network_running(name):
        start_network(name)
        if not is_network_running(name):
            raise FailedToStart(name)


@retry(wait=wait_random_exponential(multiplier=1, max=10))
def ensure_network_is_stopped(name):
    if is_network_running(name):
        stop_network(name)
        if is_network_running(name):
            raise FailedToStop(name)



from collections import defaultdict
import multiprocessing
import os
import pty
import socket
import subprocess
import time

from cuwb_stream.collector import CUWBCollector
from cuwb_stream.tools.__main__ import get_local_ip


REPO_NAME = os.getenv("REPO_NAME")

cdp_socket_ip = "0.0.0.0"
cdp_socket_port = 7667


def run_collector(message_list):
    try:
        with CUWBCollector(
            ip=cdp_socket_ip, port=cdp_socket_port, interface=get_local_ip(), timeout=2
        ) as collector:
            for msg in collector:
                message_list.append(msg)
    except socket.timeout:
        pass


def test_collector():
    test_dir = os.path.dirname(os.path.realpath(__file__))
    _, tty = pty.openpty()

    # Build the collector
    return_code = subprocess.call(
        ["make", "build-cdp-player", f"REPO_NAME={REPO_NAME}"],
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        cwd=f"{test_dir}/..",
    )
    if return_code != 0:
        raise Exception(
            "Unable to build cdp-player container, required to test UWB parsing logic"
        )

    manager = multiprocessing.Manager()
    message_list = manager.list()

    # Start the collector in a bkgd process
    p = multiprocessing.Process(target=run_collector, args=(message_list,))
    p.start()

    # Give the collector plenty of time to startup
    time.sleep(1)

    # Launch the cdp-player to stream data, the collector should capture this
    subprocess.Popen(
        ["make run-cdp-player", f"REPO_NAME={REPO_NAME}"],
        shell=True,
        stdin=tty,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        cwd=f"{test_dir}/..",
    )

    # The collector will timeout once it no longer is receiving messages
    p.join()

    assert len(message_list) == 120
    assert message_list[0]["type"] == "device_activity_state"
    assert message_list[0]["serial_number"] == "01:04:062C"

    group_messages = defaultdict(list)
    for message in message_list:
        group_messages[message["type"]].append(message)

    assert len(group_messages["anchor_health"]) == 24
    assert len(group_messages["device_activity_state"]) == 66
    assert len(group_messages["names"]) == 30

    subprocess.Popen(
        ["docker", "stop", "cdp-player"], shell=True, stdin=tty, stdout=tty, stderr=tty
    )



import click

from capture import upload_loop, capture_loop


@click.command()
@click.option('-a', '--action', help='action to take', required=True)
def main(action):
    if action=="upload":
        upload_loop()
    if action=="capture":
        capture_loop()


if __name__ == '__main__':
    main()  # pylint: disable=E1120



import datetime
import logging
from multiprocessing import Process, Queue
import os
from pathlib import Path
import time

import yaml
from minio import Minio
from minio.error import MinioException
from pythonjsonlogger import jsonlogger
from influx_line_protocol import Metric

logger = logging.getLogger()


logger.setLevel(level=logging.DEBUG)
logHandler = logging.StreamHandler()
formatter = jsonlogger.JsonFormatter('%(levelname)s %(asctime)s %(filename)s %(funcName)s %(message)s')
logHandler.setFormatter(formatter)
logger.addHandler(logHandler)


try:
    import picamera
except ImportError:
    logger.warning("picamera not available")

import ffmpeg


with open('/boot/wildflower-config.yml', 'r', encoding="utf-8") as fp:
    config = yaml.safe_load(fp.read())


DEVICE_ID = config.get("device_id", "unknown")
BUCKET = os.environ.get("MINIO_BUCKET_NAME", "videos")

CAMERA_RES = (os.environ.get("CAMERA_RES_W", 1296), os.environ.get("CAMERA_RES_H", 972))
CAMERA_FRAMERATE = os.environ.get("CAMERA_FRAMERATE", 10)
INTRA_PERIOD = os.environ.get("INTRA_PERIOD", 120)
BITRATE = os.environ.get("BITRATE", 1000000)
PERIOD = os.environ.get("DURATION", 10)
CAMERA_ISO_SETTING = int(os.environ.get("CAMERA_ISO_SETTING", 400))
CAMERA_EXPOSURE_MODE = os.environ.get("CAMERA_EXPOSURE_MODE", 'sports')
CAMERA_SHUTTER_SPEED = int(os.environ.get("CAMERA_SHUTTER_SPEED", 0))
CAMERA_H_FLIP = os.environ.get("CAMERA_H_FLIP", 0)
CAMERA_V_FLIP = os.environ.get("CAMERA_V_FLIP", 0)
METRICS_LOG_DIR = Path(os.environ.get("METRICS_LOG", "/var/log/wf_metrics"))

def get_logger(name):
    logger = logging.getLogger("wf_metrics")
    METRICS_LOG_DIR = Path(os.environ.get("METRICS_LOG", "./wf_metrics"))
    if not METRICS_LOG_DIR.exists():
        METRICS_LOG_DIR.mkdir()
    path = METRICS_LOG_DIR / f"{name}.log"
    logger.setLevel(logging.INFO)
    handy = RotatingFileHandler(path, maxBytes=20000000, backupCount=5)
    handy.setFormatter(logging.Formatter())
    logger.addHandler(handy)
    return logger

def emit(metlog, name, values, tags=None):
    metric = Metric(name)
    client.metric(name, values, tags=tags)
    metric.with_timestamp(time.time() * 1000000000)
    for tag in tags:
        metric.add_tag(tag, tags[tag])
    for value in values:
        metric.add_tag(value, values[value])
    metlog.info(metric)


def next_timeslot(now):
    return now + (PERIOD - (now % PERIOD))


def capture_loop():
    logger.setLevel(level=logging.DEBUG)
    logger.info("starting capture")
    try:
        with picamera.PiCamera() as camera:
            camera.resolution = CAMERA_RES
            camera.framerate = CAMERA_FRAMERATE
            camera.iso = CAMERA_ISO_SETTING
            camera.exposure_mode = CAMERA_EXPOSURE_MODE
            camera.shutter_speed = CAMERA_SHUTTER_SPEED
            logger.info("period is %s", PERIOD)
            camera.hflip = CAMERA_H_FLIP == "yes"
            camera.vflip = CAMERA_V_FLIP == "yes"

            now = time.time()
            timeslot = next_timeslot(now)
            sleep_time = timeslot - now

            if sleep_time < 2:  # Ensure camera has enough time to adjust
                sleep_time += PERIOD
                timeslot += PERIOD
            logger.info("going to sleep for a bit %s", sleep_time)
            time.sleep(sleep_time)
            metlog = get_logger("capture_loop")
            video_start_time = datetime.datetime.fromtimestamp(timeslot)
            name = f'/out/video-{video_start_time:%Y_%m_%d_%H_%M-%S}.h264'
            camera.start_recording(name, format='h264', intra_period=INTRA_PERIOD, bitrate=BITRATE)
            emit(metlog, "camera_capture", {"video_start": 1, "origin": 1, "split": 0})
            camera.wait_recording(PERIOD - 0.001)
            while True:
                timeslot += PERIOD
                video_start_time = datetime.datetime.fromtimestamp(timeslot)
                name = f'/out/video-{video_start_time:%Y_%m_%d_%H_%M-%S}.h264'
                logger.info("recording started", extra={"metric_tag": "video_start", "video_start_time": f"{video_start_time:%Y_%m_%d_%H_%M-%S}"})
                emit(metlog, "camera_capture", {"video_start": 1, "origin": 0, "split": 1})
                camera.split_recording(name, format='h264', intra_period=INTRA_PERIOD, bitrate=BITRATE)
                delay = timeslot + PERIOD - time.time()
                logger.info("waiting %s", delay)
                camera.wait_recording(delay)
    except Exception as e:
        logger.error(e)


def get_next_file():
    for item in os.listdir('/out'):
        if item.endswith(".h264"):
            fname = f'/out/{item}'
            st = os.stat(fname)
            if (time.time() - st.st_mtime) > 11 and st.st_size > 10000:
                return fname
    return None


def minio_client():
    KEY = os.environ.get("MINIO_KEY", "wildflower-classroom")
    SECRET = os.environ.get("MINIO_SECRET")
    if "MINIO_SERVICE_HOST" in os.environ:
        HOST = os.environ["MINIO_SERVICE_HOST"]
        PORT = os.environ["MINIO_SERVICE_PORT"]
        return  Minio(
                    ":".join([HOST, PORT]),
                    access_key=KEY,
                    secret_key=SECRET,
                    secure=False,
                )
    return  Minio(
                    os.environ["MINIO_URL"],
                    access_key=KEY,
                    secret_key=SECRET,
                    secure=False,
                )


def upload_loop():
    logger.info(os.environ["MINIO_URL"])
    minioClient = minio_client()
    try:
        minioClient.make_bucket(BUCKET, location="us-east-1")
    except MinioException:
        pass
    metlog = get_logger("upload_loop")
    while True:
        name = get_next_file()
        if name:
            try:
                start = datetime.datetime.now()
                mp4_name = f"{name[:-5]}.mp4"
                try:
                    (
                        ffmpeg
                        .input(name, format="h264", r=str(CAMERA_FRAMERATE))
                        .output(mp4_name, **{"format": "mp4", "c:v": "copy", "r": str(CAMERA_FRAMERATE)})
                        .run(quiet=True, overwrite_output=True)
                    )
                except Exception as ffmpeg_err:
                    logger.exception(ffmpeg_err)
                    logger.exception("ffmpeg_err %s", name)
                dur = (datetime.datetime.now() - start).total_seconds()
                logger.info('repackage took %s', dur)
                time.sleep(1)
                ts = name[11:-5]
                obj_name = (f'{DEVICE_ID}/{ts}.mp4').replace("_", "/")
                logger.info("putting %s on minio", obj_name)
                minioClient.fput_object(BUCKET, obj_name, mp4_name, content_type='video/mp4', metadata={
                                        "source": DEVICE_ID,
                                        "ts": ts,
                                        "duration": f"{PERIOD}s",
                                        })
                moved = 1
                logger.info("upload successful", extra={"metric_tag": "video_moved", "video_start_time": ts})
                os.remove(name)
                os.remove(mp4_name)
            except Exception as err:
                moved = 0
                logger.exception(err)
                logger.exception("failed to process %s", name)
            emit(metlog, "camera_move", {"ffmpeg_time_seconds": dur, "moved_to_control": moved})
        else:
            time.sleep(2)



import io
import logging
import socketserver
from threading import Condition
from http import server

import picamera


PAGE="""\
<html>
<head>
<title>picamera MJPEG streaming demo</title>
</head>
<body>
<h1>PiCamera MJPEG Streaming Demo</h1>
<img src="stream.mjpg" width="640" height="480" />
</body>
</html>
"""

class StreamingOutput:
    def __init__(self):
        self.frame = None
        self.buffer = io.BytesIO()
        self.condition = Condition()

    def write(self, buf):
        if buf.startswith(b'\xff\xd8'):
            # New frame, copy the existing buffer's content and notify all
            # clients it's available
            self.buffer.truncate()
            with self.condition:
                self.frame = self.buffer.getvalue()
                self.condition.notify_all()
            self.buffer.seek(0)
        return self.buffer.write(buf)


class StreamingHandler(server.BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path == '/':
            self.send_response(301)
            self.send_header('Location', '/index.html')
            self.end_headers()
        elif self.path == '/index.html':
            content = PAGE.encode('utf-8')
            self.send_response(200)
            self.send_header('Content-Type', 'text/html')
            self.send_header('Content-Length', len(content))
            self.end_headers()
            self.wfile.write(content)
        elif self.path == '/stream.mjpg':
            self.send_response(200)
            self.send_header('Age', 0)
            self.send_header('Cache-Control', 'no-cache, private')
            self.send_header('Pragma', 'no-cache')
            self.send_header('Content-Type', 'multipart/x-mixed-replace; boundary=FRAME')
            self.end_headers()
            try:
                while True:
                    with output.condition:
                        output.condition.wait()
                        frame = output.frame
                    self.wfile.write(b'--FRAME\r\n')
                    self.send_header('Content-Type', 'image/jpeg')
                    self.send_header('Content-Length', len(frame))
                    self.end_headers()
                    self.wfile.write(frame)
                    self.wfile.write(b'\r\n')
            except Exception as e:
                logging.warning(
                    'Removed streaming client %s: %s',
                    self.client_address, str(e))
        else:
            self.send_error(404)
            self.end_headers()

class StreamingServer(socketserver.ThreadingMixIn, server.HTTPServer):
    allow_reuse_address = True
    daemon_threads = True

with picamera.PiCamera(resolution='640x480', framerate=24) as camera:
    output = StreamingOutput()
    camera.start_recording(output, format='mjpeg')
    try:
        address = ('', 8000)
        server = StreamingServer(address, StreamingHandler)
        server.serve_forever()
    finally:
        camera.stop_recording()



from datetime import datetime
import logging
import os
import socket

import cdp


POSITION_V3 = 0x0135
ACCELEROMETER_V2 = 0x0139
GYROSCOPE_V2 = 0x013A
MAGNETOMETER_V2 = 0x013B
PRESSURE_V2 = 0x013C
QUATERNION_V2 = 0x013D
TEMPERATURE_V2 = 0x013E
DEVICE_NAMES = 0x013F
HARDWARE_STATUS_V2 = 0x0138
ANCHOR_HEALTH_V5 = 0x014A
NETWORK_TIME_MAPPING_V1 = 0x015A
DEVICE_ACTIVITY_STATE = 0x0137

MAX_TIMESTAMP_REFRESH_DELAY = (
    60  # seconds before we lose confidence in our interpolated real time value
)


DEBUG = os.getenv("DEBUG", "False").lower() in ("true", "1", "t")


class CUWBCollector:
    def __init__(self, ip, port, interface, timeout=None):
        self.ip = ip
        self.port = port
        self.interface = interface
        self.listen_socket = None
        self.timeout = timeout

    def start(self):
        if not self.listen_socket:
            self.listen_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            self.listen_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.listen_socket.bind((self.ip, self.port))
            if self.timeout:
                self.listen_socket.settimeout(self.timeout)
            try:
                self.listen_socket.setsockopt(
                    socket.SOL_IP,
                    socket.IP_ADD_MEMBERSHIP,
                    socket.inet_aton(self.ip) + socket.inet_aton(self.interface),
                )
            except socket.error:
                logging.warning(
                    "Failed connecting to socket through remote IP, falling back to a non-routed local connection"
                )

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, _type, value, traceback):
        self.listen_socket.close()
        self.listen_socket = None

    def extract_data_items(
        self,
        socket_read_time,
        data_item_type,
        type_name,
        cdp_packet,
        fields,
        debug=False,
    ):
        for item in cdp_packet.data_items_by_type.get(data_item_type, []):
            if debug:
                logging.warning(
                    f"Logging {type_name}: Definition: {item.definition} - Full Item: {item}"
                )

            timestamp = socket_read_time

            data = {
                "socket_read_time": socket_read_time,
                "timestamp": timestamp,
                "type": type_name,
            }
            for field in fields:
                if hasattr(item, field):
                    data[field] = getattr(item, field)
                    if field == "serial_number":
                        data[field] = str(data[field])
                    if field == "bad_paired_anchors":
                        data[field] = ",".join([str(di) for di in data[field]])

            yield data

    def __iter__(self):
        self.start()

        while True:
            socket_read_time = None

            try:
                data, _ = self.listen_socket.recvfrom(65535)

                socket_read_time = datetime.utcnow()
                cdp_packet = cdp.CDP(data)
            except ValueError:
                logging.error("Failed parsing socket content")
                yield None
            except socket.timeout:
                logging.error("Socket timed out")
                return

            logging.info(f"Packet received at: {socket_read_time}")

            fields = [
                "serial_number",
                "network_time",
                "x",
                "y",
                "z",
                "scale",
            ]
            for item in self.extract_data_items(
                socket_read_time,
                ACCELEROMETER_V2,
                "accelerometer",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item
            for item in self.extract_data_items(
                socket_read_time,
                GYROSCOPE_V2,
                "gyroscope",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item
            for item in self.extract_data_items(
                socket_read_time,
                MAGNETOMETER_V2,
                "magnetometer",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item

            fields = [
                "serial_number",
                "network_time",
                "x",
                "y",
                "z",
                "w",
                "quaternion_type",
            ]
            for item in self.extract_data_items(
                socket_read_time,
                QUATERNION_V2,
                "quaternion",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item

            for item in self.extract_data_items(
                socket_read_time,
                PRESSURE_V2,
                "pressure",
                cdp_packet,
                ["serial_number", "network_time", "pressure", "scale"],
                debug=DEBUG,
            ):
                yield item

            for item in self.extract_data_items(
                socket_read_time,
                TEMPERATURE_V2,
                "temperature",
                cdp_packet,
                ["serial_number", "network_time", "temperature", "scale"],
                debug=DEBUG,
            ):
                yield item

            for item in self.extract_data_items(
                socket_read_time,
                DEVICE_NAMES,
                "names",
                cdp_packet,
                ["serial_number", "name"],
                debug=DEBUG,
            ):
                yield item

            fields = [
                "serial_number",
                "network_time",
                "x",
                "y",
                "z",
                "anchor_count",
                "quality",
                "flags",
                "smoothing",
            ]
            for item in self.extract_data_items(
                socket_read_time,
                POSITION_V3,
                "position",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item

            fields = [
                "serial_number",
                "memory",
                "flags",
                "minutes_remaining",
                "battery_percentage",
                "temperature",
                "processor_usage",
            ]
            for item in self.extract_data_items(
                socket_read_time,
                HARDWARE_STATUS_V2,
                "status",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item

            fields = [
                "serial_number",
                "interface_id",
                "ticks_reported",
                "timed_rxs_reported",
                "beacons_reported",
                "beacons_discarded",
                "beacons_late",
                "average_quality",
                "report_period",
                "interanchor_comms_error_code",
                "bad_paired_anchors",
            ]
            for item in self.extract_data_items(
                socket_read_time,
                ANCHOR_HEALTH_V5,
                "anchor_health",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item

            fields = [
                "serial_number",
                "interface_id",
                "x",
                "y",
                "z",
                "role_id",
                "connectivity_state",
                "synchronization_state",
            ]
            for item in self.extract_data_items(
                socket_read_time,
                DEVICE_ACTIVITY_STATE,
                "device_activity_state",
                cdp_packet,
                fields,
                debug=DEBUG,
            ):
                yield item


def network_time_to_seconds(ut):
    return float(ut) * 15.65 / (1e12)



import json


class DatabaseConnectionSnoop:
    def write_datapoint_object_time_series(self, object_id, data):
        value_dict = {"object_id": object_id}
        value_dict.update(data)
        value_dict["timestamp"] = (
            value_dict["timestamp"].isoformat()
            if value_dict.get("timestamp", None)
            else None
        )
        value_dict["socket_read_time"] = (
            value_dict["socket_read_time"].isoformat()
            if value_dict.get("socket_read_time", None)
            else None
        )
        print(json.dumps(value_dict))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['capture', 'cuwb_stream', 'cuwb_stream.tests', 'cuwb_stream.tools']

package_data = \
{'': ['*']}

install_requires = \
['ffmpeg-python==0.2.0',
 'minio>=7.1.12',
 'python-json-logger>=2.0.4',
 'pyyaml>=6.0']

setup_kwargs = {
    'name': 'wf-camera-utils',
    'version': '1.0.20',
    'description': 'utilities for wildflower cameras and raspberry pi',
    'long_description': "# Camera-node\n\nAll software needed for a camera node in an environment. Includes pieces for both capture and delivery as well as proxy node that handles centralized processing of video, upload, and queuing in case of slow or down connectivity.\n\n## Proxy\n\nFlask service that receives data from all other camera nodes and feeds and internal queue and that sends data to honeycomb and optionally processes videos to pull out frames that are also sent to honeycomb.\n\n## Capture\n\nService that runs on nodes that have cameras. Captures video clips in short segments and sends them to their internal queue where workers send the to the proxy.\n\n## Workers\n\nCelery service that performs tasks from the internal queue. Capture work is processed and sent to the proxy, unless it is a the proxy-node in which case it forwards the work to another task. Video segments are processes and sent to honeycomb. Processing includes tagging etc. Once uploaded the next set of processing happens, key frames are extracted at those are sent to honeycomb as child datapoint objects of the video.\n\nIf running on more capable hardware the keyframes could be evaluated for things like pose detection or object tracking. This is not planned yet but isn't out of scope specifically.\n\n## Radio-Monitor\n\nA python service that connects to a network of DWM1001 modules over BLE to collect data. That data is queued to be sent to honeycomb. It is expected that this service runs on the proxy node.\n\n## CUWB-Stream\n\nLeverages fluentd to move Ciholas sensor data to S3\n\nBefore deploying the service, update the Ciholas network config as follows:\n\n|        | IP                | Port   | Interface           |\n|--------|-------------------|--------|---------------------|\n| Config | 239.255.76.67     | 7671   | <<CONTROL PC IP>>   |\n| Input  | 239.255.76.67     | 7667   | <<CONTROL PC IP>>   |\n| Output | 0.0.0.0           | 32222  | <<CONTROL PC IP>>   |\n\nPS: In order to resolve an issue with the anchors disconnecting and not reconnecting, you may need to set the interface IPs of the Config and Input rows to the ethernet device's IP (Use `ifconfig`) \n\n### Build and push cuwb service\n\n```\nmake build-cuwb-stream\n```\n\n### Deploy streaming service to k8\n\n```\n# Install envsubtr, on MacOS install through the gettext pkg\nbrew install gettext\nbrew link --force gettext \n\n# Create a config and secrets file with S3 and AWS ENV keys\nkubectl apply -f ./k8s/kube-logging.yml\nkubectl apply -f ./private/aws-s3-write-auth-config.yml\nkubectl apply -f ./private/aws-s3-write-auth-secret.yml\nkubectl apply -f ./k8s/fluentd.yml\nkubectl apply -f ./k8s/fluentd-s3-config.yml\nkubectl apply -f ./k8s/fluentd-s3.yml\n\nTIMEZONE=US/Pacific envsubst < ./k8s/fluentd-s3-scheduler.yml | kubectl apply -f -\n\nkubectl apply -f ./k8s/cuwb-service.yml \n```\n\n### Test CUWB Steaming logger\n\nSee the [cdp_player README](./cuwb_stream/README.md#Test)\n\n## CDP Player\n\nYou can use the Makefile to build or run the cdp-player. You can also [work with cdp-player directly](./cdp_player/README.md).\n\n### Build\n\n`make build-cdp-player REPO_NAME=<<Ciholas PPA Repository Name>>`\n\n### Run \n\n`make run-cdp-player REPO_NAME=<<Ciholas PPA Repository Name>>`\n\n## Setup cluster with Docker Hub robot\n\nFirst login and then copy creds into the cluster:\n\n    docker login\n    # Provide username and PAT (personal access token)\n\n    kubectl create secret generic regcred --from-file=.dockerconfigjson=/home/wildflowertech/.docker/config.json --type=kubernetes.io/dockerconfigjson\n\n\n## Logz + Fluentd Experiment\n\n    kubectl create namespace monitoring\n\n    kubectl create secret generic logzio-logs-secret \\\n        --from-literal=logzio-log-shipping-token='<<REDACTED>>' \\\n        --from-literal=logzio-log-listener='https://listener.logz.io:8071' \\\n        -n monitoring\n\n    # fluentd-general-config.yml contains the CLASSROOM_ENVIRONMENT env var\n    kubectl apply -f ./private/fluentd-general-config.yml\n    kubectl apply -f ./k8s/fluentd-general-config.yml -f ./k8s/fluentd-general-monitoring.yml\n",
    'author': 'Paul J DeCoursey',
    'author_email': 'paul@decoursey.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import os
import socket

import click

from cuwb_stream.collector import CUWBCollector
from cuwb_stream.network import ensure_network_is_running, ensure_network_is_stopped
from cuwb_stream.tools.snooplogg import DatabaseConnectionSnoop


def get_local_ip(routable_ip="8.8.8.8"):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect((routable_ip, 80))
    ip = s.getsockname()[0]
    s.close()
    return ip


@click.group()
@click.pass_context
def main(ctx):
    pass


@main.command()
@click.pass_context
@click.option(
    "--consumer",
    help="the consumer type, supports `stdout`, `csv`, or `honeycomb` at this time",
)
@click.option(
    "--environment_name_honeycomb",
    help="name of the environment in honeycomb, required for using the honeycomb consumer",
)
@click.option(
    "--path_csv", help="path for csv file output, required for using the csv consumer"
)
@click.option(
    "--ip",
    help="Socket ip for the CUWB network. (defaults to CUWB_SOCKET_IP env or 239.255.76.67)",
)
@click.option(
    "--port",
    help="Socket port for the CUWB network. (defaults to CUWB_SOCKET_PORT env or 7667)",
)
@click.option(
    "--route_ip",
    help="IP that the interface should route to. (defaults to CUWB_SOCKET_PORT env or 8.8.8.8)",
)
def collect(
    ctx,
    consumer,
    environment_name_honeycomb=None,
    path_csv=None,
    ip=None,
    port=None,
    route_ip=None,
):
    if consumer not in ["stdout"]:
        raise ValueError("consumer must be `stdout`")

    database_connection = DatabaseConnectionSnoop()

    ip = ip if ip else os.environ.get("CUWB_SOCKET_IP", "0.0.0.0")
    port = port if port else os.environ.get("CUWB_SOCKET_PORT", 7667)
    interface = (
        route_ip
        if route_ip
        else get_local_ip(os.environ.get("CUWB_ROUTABLE_IP", "8.8.8.8"))
    )
    with CUWBCollector(ip, int(port), interface) as collector:
        for bit in collector:
            if bit:
                database_connection.write_datapoint_object_time_series(
                    object_id=bit.get("serial_number"), data=bit
                )


@main.command()
@click.pass_context
@click.option("--name", help="name of the network")
@click.option("--action", help="start or stop")
def network(ctx, name, action):
    if action == "start":
        ensure_network_is_running(name)
    elif action == "stop":
        ensure_network_is_stopped(name)


@main.command()
@click.pass_context
@click.option("--file", help="path to the file to upload")
@click.option("--serial_number", help="serial_number for the device")
@click.option("--start", help="starting timestamp")
@click.option("--type", help="type of measurement(s)")
@click.option(
    "--environment_name_honeycomb",
    help="name of the environment in honeycomb, required for using the honeycomb consumer",
)
def upload(ctx, environment_name_honeycomb, file, serial_number, start, type):
    from database_connection_honeycomb import DatabaseConnectionHoneycomb

    database_connection = DatabaseConnectionHoneycomb(
        environment_name_honeycomb=environment_name_honeycomb,
        object_type_honeycomb="DEVICE",
        object_id_field_name_honeycomb="serial_number",
    )
    with open(file, "r") as fp:
        database_connection.write_datapoint_object_time_series(
            timestamp=start, object_id=serial_number, data=fp.read()
        )


if __name__ == "__main__":
    main()
